#include <iostream>
#include <iomanip>
#include "BeetleConfig.h"

static const char *var_name[] =
{
    "Itp","Ipre","Isha","Ibuf","Vfp","Vfs","Vrc","Ipipe","Vd",
    "Vdcl","Ivoltbuf","Isf","Icurrbuf","Latency","ROCtrl","RclkDiv",
    "CompCtrl",
    "Icomp", "Ithdelta", "Ithmain",
    0
};
static const unsigned short var_value[] =
{
    0x0000, 0x014c, 0x020A, 0x030a, 0x0400, 0x0500, 0x0900,
    0x0a0d, 0x0b82, 0x0c69, 0x0d14, 0x0e1a, 0x0f66, 0x1080,
    0x111a, 0x1200, 0x1309,
    0x0605, 0x0700, 0x0800
};

static unsigned short var_index[sizeof(var_value)/sizeof(unsigned short)] = {0xffff};

BeetleConfig::BeetleConfig() : _mask(3)
{
    int i;
    set_group("beetle");
    for (i=0; var_name[i]; i++)
        (*this)[var_name[i]] = var_value[i];

    if ( var_index[0] == 0xffff )
    {
        for (i=0; var_name[i]; i++)
            var_index[i] = (var_value[i] & 0xff00);
    }
}

unsigned short BeetleConfig::addr(int i) const
{
    return var_index[i];
}

void BeetleConfig::cpy(const BeetleConfig &b)
{
    clear();
    const_iterator ip;
    for (ip=b.begin(); ip!=b.end(); ++ip)
    {
        (*this)[ip->first] = ip->second;
    }
    _mask = b._mask;
    set_group(b.get_group() );
}

void BeetleConfig::load_data()
{
    if (exists())
    {
        int i;
        for (i=0; var_name[i]; i++)
        {
            (*this)[var_name[i]] = (var_value[i] & 0xff00) | (0xff &get_int(var_name[i]));
        }
        _mask = get_int("mask", 3);
    }
}
void BeetleConfig::save_data()
{
    iterator ip;
    for (ip=begin(); ip!=end(); ++ip)
    {
        set_int(ip->first, (0xff & ip->second));
    }
    set_int("mask", _mask);
}

void BeetleConfig::get_comparator_settings(ComparatorConfig &comparator)
{
    unsigned short CompCtrl = (*this)["CompCtrl"] & 0xff;
    unsigned short Ithmain = (*this)["Ithmain"] & 0xff;
    unsigned short Ithdelta = (*this)["Ithdelta"] & 0xff;
    unsigned short Vrc = (*this)["Vrc"] & 0xff;

    comparator.setEnable( (CompCtrl & 9)==0 );
    comparator.setPolarity( (CompCtrl & 2)/2 );
    comparator.setMode( (CompCtrl & 16) ? ComparatorConfig::pulseMode : ComparatorConfig::trackMode );
    comparator.setMainTh( Ithmain );
    comparator.setDeltaTh( Ithdelta );
    comparator.setVrc( Vrc );
}

void BeetleConfig::set_comparator_settings(const ComparatorConfig &comparator)
{
    unsigned short ctrl = 0;
    if ( !comparator.isEnabled() )
    {
        ctrl = 9;
    }
    if ( comparator.getPolarity() )
        ctrl |= 2;

    if ( comparator.getMode() == ComparatorConfig::pulseMode )
        ctrl |= 16;


    unsigned short CompCtrl = (*this)["CompCtrl"] & 0xff00;
    unsigned short Ithmain = (*this)["Ithmain"] & 0xff00;
    unsigned short Ithdelta = (*this)["Ithdelta"] & 0xff00;
    unsigned short Vrc = (*this)["Vrc"] & 0xff00;

    (*this)["CompCtrl"] = CompCtrl | ctrl;
    (*this)["Ithmain"] = Ithmain | comparator.getMainTh();
    (*this)["Ithdelta"] = Ithdelta | comparator.getDeltaTh();
    (*this)["Vrc"] = Vrc | comparator.getVrc();
}
