/*
 * ConfigFactory.cc
 *
 *  Created on: May 17, 2010
 *      Author: lacasta
 */

#include "ConfigFactory.h"
#include <PluginConfig.h>
#include <ScanConfig.h>
#include <BeetleConfig.h>
#include <LaserConfig.h>
#include <TriggerConfig.h>
#include <DAQConfig.h>
#include <ComparatorConfig.h>
#include <Analysis.h>
#include <Alibava.h>
#include "utils.h"

typedef std::map<std::string, ConfigFile *> ConfigMap;
static ConfigFactory *__the_factory__ = 0;

ConfigFactory::ConfigFactory()
{
    config_map["calibration"] = new CalibrationConfig(50, 0, 100000);
    config_map["lasersync"] = new LaserSyncConfig(100, 0, 1000);
    config_map["beetle"] = new BeetleConfig();
    config_map["laser"] = new LaserConfig();
    config_map["trigger"] = new TriggerConfig();
    config_map["daq"] = new DAQConfig();
    config_map["plugin"] = new PluginConfig();
    config_map["analysis"] = new Analysis();
    config_map["comparator"] = new ComparatorConfig;
}

ConfigFactory::~ConfigFactory()
{
    ConfigMap::iterator ip;
    for (ip = config_map.begin(); ip!=config_map.end(); ip++)
    {
        delete ip->second;
    }
    config_map.clear();
}

ConfigFactory &ConfigFactory::instance()
{
    if (!__the_factory__)
        __the_factory__ = new ConfigFactory();

    return *__the_factory__;
}

void ConfigFactory::set_file(KeyFile *key_file)
{
    ConfigMap::iterator ip;
    for (ip = config_map.begin(); ip!=config_map.end(); ip++)
    {
        ip->second->set_file(key_file);
    }
}

void ConfigFactory::save_data()
{
    ConfigMap::iterator ip;
    for (ip = config_map.begin(); ip!=config_map.end(); ip++)
    {
        ip->second->save_data();
    }
}

ConfigFile *ConfigFactory::get(const std::string &name)
{
    ConfigMap::iterator ip = config_map.find(name);
    if (ip!=config_map.end())
        return ip->second;
    else
        return 0;
}
int ConfigFactory::config_beetle(Alibava &B)
{
    write_msg(_("*** Sending Beetle configuration\n"));
    BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(ConfigFactory::instance().get("beetle"));
    unsigned short *val = new unsigned short[beetle->size() ];
    BeetleConfig::iterator ip;
    int nval=0;

    for (ip=beetle->begin(); ip!=beetle->end(); ++ip)
    {
        val[nval++] = ip->second;
    }

    if (B.beetle_config(nval, beetle->mask(), val)<0)
    {
        delete [] val;
        return -1;
    }
    delete [] val;
    return 0;

}
int ConfigFactory::config_trigger(Alibava &B)
{
    write_msg(_("*** Sending Trigger configuration\n"));
    TriggerConfig *tc = dynamic_cast<TriggerConfig *>(ConfigFactory::instance().get("trigger"));

    // Thresholds go in DAC units, so we have to translate
    return B.trigger(tc->pulse() ? Alibava::TriggerPulse : Alibava::TriggerIn,
                          tc->trigger_and() ? Alibava::TriggerAND : Alibava::TriggerOR,
                          ((tc->dacA()+2048) & 0xfff),
                          ((tc->dacB()+2048) & 0xfff)| 0x1000,
                          ((tc->dacC()+2048) & 0xfff)| 0x2000,
                          ((tc->dacD()+2048) & 0xfff)| 0x3000
                          );
}

int ConfigFactory::config_board(Alibava &B)
{
    int rc;


    DAQConfig *daq = dynamic_cast<DAQConfig *>(get("daq"));
    B.delay(daq->delay());


    if ( (rc=config_beetle(B)) )
        return rc;

    rc = config_trigger(B);
    return rc;
}
