#include <glib.h>
#if HAVE_CONFIG_H
#ifdef G_OS_WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif

#include <iostream>
#include <string>
#include <sstream>
#include <cstdlib>
#include <cstring>
#include <csignal>
#include <getopt.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "ConfigFactory.h"
#include "AsciiDataFile.h"
#include "DAQmanager.h"
#include "AlibavaGUI.h"
#include "USBserial.h"
#include "USBemulator.h"
#include "USBFifo.h"
#include "USBserialWnds.h"
#include "Alibava.h"
#include "AlibavaMonitor.h"
#include "Progress.h"
#include "utils.h"
#include "path_utils.h"
#include "alibava_glade.h"
#include "alibava_icon.h"

#ifdef USE_SOAP
#include <AlibavaSOAP.h>
#endif
AlibavaGUI *the_daq_manager=0;

#ifdef G_OS_WIN32
void try_to_get_windows_font(void);
gchar *get_registry_key(const gchar *path, const gchar *key);
#endif

#ifdef __APPLE__
#include <gtkosxapplication.h>
GtkosxApplication *theMacApp=NULL;
#endif

#ifdef HAS_FTD2XX
#include "USBd2xx.h"
#endif
//#define USE_EMULATOR
//#define USE_FIFO


void test_emitter(GObject *obj, char *group, char *key, gpointer data)
{
    AlibavaGUI *A = (AlibavaGUI *)data;
    ConfigFactory &cfg = ConfigFactory::instance();

    if ( !strcmp(group, "DAQ"))
    {
        DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg.get("daq"));
        if ( !strcmp(key, "nevts") )
        {
            A->int2entry("entryMaxEvents", daq->nevts() );
        }
    }
    else if ( !strcmp(group, "laser"))
    {
        LaserConfig *laser = dynamic_cast<LaserConfig *>(cfg.get("laser"));
        if (!strcmp(key, "delay"))
        {
            A->int2entry("txtDelay", laser->delay() );
        }
    }

}


DAQmanager *__mngr__=0;
void INTcatcher(int sig)
{
    printf("INTcatcher\n");
    if (sig == SIGINT)
    { // if single control-C, just set flag to stop run
        printf("...SIGINT\n");
        if (__mngr__)
        {
            printf("\nAaaarrrrrggggg !!!!!\n");
            __mngr__->stop_run();
        }
    }
}


extern int __is_gui_session__;
// User data folder
const gchar *user_data_dir=0;

#if USE_INTLTOOL
#include <locale.h>
class LocaleSetter
{
    public:
        LocaleSetter()
        {
#ifdef G_OS_WIN32
            gchar *LOCALEDIR=get_registry_key("SOFTWARE\\AlibavaSL", "LOCALEDIR");
#endif
//#ifdef G_OS_UNIX
            setlocale (LC_ALL, "");
            bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
            bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
            textdomain (GETTEXT_PACKAGE);

            gchar *locale = getenv("LANG");
            if (locale)
                locale = g_strdup(locale);
            else
                locale = g_strdup("");

            //printf("locale is %s\n", setlocale(LC_ALL, locale));
            gtk_disable_setlocale();
            g_free(locale);
//#endif
#ifdef G_OS_WIN32
            g_free(LOCALEDIR);
#endif
        }
};
static LocaleSetter __set_locale__;
#endif

static char *glade_file=0;
const char *get_glade_file()
{
    return glade_file;
}

class MyReceiver : public DataReceiver
{
    private:
        int nevts;
        Progress prog;
    public:
        MyReceiver(int nn) :
            nevts(nn),
            prog(0, 1000, Progress::show_status | Progress::show_rate
                    | Progress::show_left | Progress::show_time | Progress::show_memory
                    | Progress::show_value)
        {
            prog.set_update(500000);
            prog.reset(1, nevts);
        }
        void receive(const EventData *)
        {
            prog.increment();
        }
        void start() { prog.start(); }
        void stop() { prog.stop(); }

};


static void my_log (const gchar *log_domain,
                    GLogLevelFlags log_level,
                    const gchar *message,
                    gpointer user_data)
{
    if (!strcmp(log_domain,"libglade") && strstr(message,"unknown"))
        return;

    printf("MyLogger: %s\n", message);
    //    (*old_log)(log_domain,log_level,message,user_data);
}
static void setup_logger()
{
    g_log_set_handler ("GLib-GObject",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("Gtk",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("Gdk",
                          (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                           G_LOG_FLAG_FATAL |
                                           G_LOG_FLAG_RECURSION),
                          my_log, 0);
    g_log_set_handler ("GLib",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("libglade",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                               G_LOG_FLAG_FATAL |
                               G_LOG_FLAG_RECURSION),
                               my_log, 0);

}

std::string find_dev_file()
{
    std::string out;
    std::vector<std::string> vout;
#ifdef __APPLE__
    PathUtils::find_file("cu.usbserial-", vout, "/dev");
#else
    PathUtils::find_file("alibava", vout, "/dev");
    #ifdef __linux__
    if (vout.empty())
        PathUtils::find_file("ttyUSB", vout, "/dev");
    #endif
#endif
    if (!vout.empty())
    {
        out = vout[0];
    }
    return out;
}


gchar *__daemon_path__ = 0;
gchar *__symbol_browser_path__ = 0;

int main(int argc, char **argv)
{
    /**
     *  Get the command line parameters
     */
    int igui=1;
    int iemulator=0;
    int idebug = 0;
    int isource = -1;
    int firmware = 1;
    int firmware_set = 0;
    int iplugin = 0;
    int ififo=0;
    int id2xx=0;
    int iverbose=0;
    int nevts=100000;
    int sample_size=-1;
    int iport=10000;
    int option_index;
    bool no_dev_found=false;
    std::string cfg_file;
    std::string out_data_file;
    std::string device;

    // Make sure glib threads are initialized
    g_type_init();
    if (!g_thread_supported ())
        g_thread_init (NULL);

#ifdef HAVE_GIO
    GError *err=0;
    char *tmp;
    std::string cwd = PathUtils::getcwd();
    std::string prog_name( PathUtils::abspath(argv[0]) );
    std::string _prog_path, _daemon_path;
    _prog_path = PathUtils::dir_name( prog_name );

#ifdef G_OS_WIN32
    _daemon_path = PathUtils::find_bin( PathUtils::join(_prog_path, "alibavad.exe" ) );
#else
    _daemon_path = PathUtils::find_bin( PathUtils::join(_prog_path, "alibavad" ) );
    if (!PathUtils::exists(_daemon_path))
    {
        char *ddd = getenv("ALIBAVA_DAEMON");
        if (ddd)
        {
            std::string tmp = PathUtils::find_bin( ddd );
            if ( PathUtils::exists(tmp) )
                _daemon_path = tmp;
        }
        else
        {
            std::vector<std::string> ofiles;
            PathUtils::find_file("alibavad", ofiles, _prog_path,false);
            if (!ofiles.empty())
                _daemon_path = ofiles[0];
        }
    }

#endif
    __daemon_path__ = g_strdup(_daemon_path.c_str());
#ifdef G_OS_WIN32
    std::string _symbol_browser = PathUtils::find_bin( PathUtils::join(_prog_path, "plugin_dump.exe"));
    __symbol_browser_path__ = g_strdup(_symbol_browser.c_str());
#endif

    g_set_application_name("AlibavaGUI");
    user_data_dir = g_build_filename( g_get_user_special_dir( G_USER_DIRECTORY_DOCUMENTS ),
                                      g_get_application_name(),
                                      NULL);
    GFile *outdir = g_file_new_for_path(user_data_dir);
    gboolean dir_made = g_file_make_directory_with_parents(outdir, NULL, &err);
    if( err && err->code != G_IO_ERROR_EXISTS )
    {
        err_msg("Cannot create data folder: %s", err->message);
        g_error_free(err);
        err = 0;
    }

#endif


    static struct option long_options[] =
         {
          { "gui", 0, &igui, 1 },
          { "debug", 0, &idebug, 1},
          { "no-gui", 0,&igui, 0 },
          { "emulator", 0, &iemulator, 1},
          { "no-emulator", 0, &iemulator,0},
          { "calib", 0,&isource, RunManager::Calibration },
          { "laser", 0,&isource, RunManager::Laser },
          { "source", 0,&isource, RunManager::RadSource },
          { "pedestal", 0,&isource, RunManager::Pedestal },
          { "activate-plugin", 0, &iplugin, 1},
          { "fifo", 0, &ififo, 1},
          { "d2xx",0,&id2xx,1},
          { "no-fifo", 0, &ififo, 0},
          { "verbose", 0, &iverbose, 1},
          { "nevts", 1, 0, 'n' },
          { "sample", 1, 0, 's'},
          { "dev", 1, 0, 'd'},
          { "firmware",1,0,'f'},
          { "out", 1, 0, 'o'},
          {"soap-port", 1, 0, 'p'},
          { "help", 0, 0, 'h'},
          { 0, 0, 0, 0 } };

    while ( 1)
    {
        char c;
        option_index = 0;
        c = getopt_long(argc, argv, ":x:o:s:r:w:h:t:ud", long_options, &option_index);
        if (c == -1)
            break;
        switch (c)
        {
            case 0:
                break;
            case 'n':
                nevts = atoi(optarg);
                break;
            case 's':
                sample_size = atoi(optarg);
                break;
            case 'f':
            	firmware_set = 1;
                firmware = atoi(optarg);
                break;
            case 'd':
                device = optarg;
                break;
            case 'o':
                out_data_file = optarg;
                break;
            case 'p':
                iport = atoi( optarg);
                break;
            case 'h':
                std::cout << "alibava-gui: XXXX" << std::endl;
                return 0;
            default:
                std::cout << "Error " << argv[optind-1] << " ";
                if (optind<argc)
                    std::cout << argv[optind];
                std::cout << " [Ughhhh !!!!]" << std::endl;
                return 1;
        }
    }
    while (optind<argc)
    {
        cfg_file = PathUtils::abspath( argv[optind++] );
    }

    if (iverbose)
        __verbose__ = 1;

    __is_gui_session__ = igui;
    if (user_data_dir)
        chdir(   user_data_dir );


#ifdef G_OS_UNIX
    if ( device.empty() && !iemulator)
    {
        device = find_dev_file();
        if (device.empty())
        {
            std::cout << "Starting in emulation mode" << std::endl;
            iemulator = 1;
            no_dev_found = true;
        }
    }
#else
    device = "XXXX";
#endif

    /*
     * Change the default for the emulator
     */
#ifdef G_OS_WIN32
    /// Emulator + Fifo do not work in windows
    if ( iemulator && (ififo == 999) )
        ififo = 0;

#endif

    setup_logger();
    /**
     * Decide which driver to use
     */
    USBport *port=0;
    USBport *usb=0;


    if (ififo)
    {
        if (iemulator)
        {
            usb = new USBemulator(firmware);
        }
        else
        {

#ifdef G_OS_UNIX
            usb = new USBserial;
#elif defined(G_OS_WIN32)
#ifdef HAS_FTD2XX
            if (id2xx)
                usb = new USBd2xx;
            else
#endif
            usb = new USBserialWnds;
#endif
        }
        if (id2xx)
        	port = usb;
        else
        	port = new USBFifo(*usb);
    }
    else
    {
        if (iemulator)
            port = new USBemulator(firmware);
        else
        {
#ifdef G_OS_WIN32
#ifdef HAS_FTD2XX
            if (id2xx)
                port = new USBd2xx;
            else
#endif
            	port = new USBserialWnds;
#else
                port = new USBserial;
#endif
        }
    }


    Alibava board(*port);
    PluginConfig *plg = (PluginConfig *)ConfigFactory::instance().get("plugin");
    plg->active( (iplugin != 0) );
    if ( firmware_set )
    	board.force_firmware(firmware);

#ifdef USE_SOAP
    // Start the SOAP server
    AlibavaSOAP *the_soap_server = new AlibavaSOAP(iport);
    the_soap_server->serve(true);
    usleep(100);
    if ( !the_soap_server->is_running() )
    {
        std::cout << "**** Could not start the SOAP server ******" << std::endl;
        if (the_soap_server->error() )
            std::cout << '\t' << the_soap_server->strerror() << std::endl;
    }
#endif


    if (!igui)
    {
        ConfigFactory &cfg = ConfigFactory::instance();
        DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg.get("daq"));
        MyReceiver R(nevts);
        DAQmanager D(nevts, &board, (RunManager::RunType)isource);
        board.set_data_receiver(&R);


        __mngr__ = &D;
        D.set_data_file( out_data_file );
        D.open(device.c_str());

        if (!cfg_file.empty())
            D.load_config(cfg_file.c_str());

        // overwrite what the config file says
        daq->nevts(nevts);
        daq->firmware( firmware );

        // no gui
        if ( D.prepare_for_run() )
            return -1;

        R.start();
        signal(SIGINT, INTcatcher);

        if ( D.acquire() )
            return -1;

        R.stop();
        D.stop_run();
    }
    else
    {
        // First thing to do is to initialize Gtk and Glade
        char glade_filename[128];
        gtk_init(&argc, &argv);
        glade_init();

#ifdef MAC_INTEGRATION
        theMacApp = (GtkosxApplication *)g_object_new(GTKOSX_TYPE_APPLICATION, NULL);
#endif


        if (no_dev_found)
        {
            GtkDialog *dlg = GTK_DIALOG( gtk_message_dialog_new_with_markup(0,
                                                                (GtkDialogFlags)(GTK_DIALOG_DESTROY_WITH_PARENT),
                                                                GTK_MESSAGE_INFO,
                                                                GTK_BUTTONS_OK,
                                                                "<b>alibava-gui</b>:\nNo devices found.\nStarting in emulation mode",
                                                                0));
            gtk_dialog_run(dlg);
            gtk_widget_hide(GTK_WIDGET(dlg));
            gtk_widget_destroy(GTK_WIDGET(dlg));
        }


        const gchar *tmp_dir = g_get_tmp_dir();
        sprintf(glade_filename,"alibava.%d.glade", getpid());
        glade_file = g_build_filename(tmp_dir, glade_filename, NULL);
        FILE *ofile = fopen(glade_file,"w");
        if ( !ofile )
        {
            err_msg(_("Cannot write alibava.glade\n"));
            return -1;
        }
        fwrite(alibava_glade, sizeof(alibava_glade), 1, ofile);
        fclose(ofile);

#ifdef G_OS_WIN32
        try_to_get_windows_font();
#endif
//        char *app_font = 0;
//        GtkSettings *settings = gtk_settings_get_default();
//        g_object_get(G_OBJECT(settings), "gtk-font-name", &app_font, NULL);
//        std::cout << "::> Aplication font " << app_font << std::endl;
//        g_free(app_font);


        AlibavaGUI window(nevts, &board);
        window.set_debug( idebug!=0 );
        the_daq_manager = &window;

        ConfigFactory &cfg = ConfigFactory::instance();
        cfg.get("daq")->connect( G_CALLBACK(test_emitter), &window);
        cfg.get("laser")->connect(G_CALLBACK(test_emitter), &window);

        if (sample_size>0)
            window.sample_size(sample_size);

        window.init(glade_file, "MainWindow");
        GdkPixbuf *pxIcon = gdk_pixbuf_new_from_inline(-1, alibava_icon, FALSE, NULL);
        g_object_set(G_OBJECT(window.get_main_window()),"icon", pxIcon,NULL);
        gtk_window_set_icon_name( GTK_WINDOW(window.get_main_window()), "alibava" );
#ifdef MAC_INTEGRATION
        pxIcon = gdk_pixbuf_new_from_inline(-1, alibava_icon_transp, FALSE, NULL);
        gtkosx_application_set_dock_icon_pixbuf(theMacApp, pxIcon);
#endif

        if (window.open(device.c_str()) )
        {
            err_msg(_("Could not open the device."));
        }
        if (!cfg_file.empty())
            window.load_config(cfg_file.c_str());


        // Now we start the Gtk loop
        gtk_main();
        unlink(glade_file);
    }
    delete port;

    std::cout.flush();
    if (usb)
        delete usb;

    return 0;
}

void set_daq_manager_data_file(const std::string &fnam)
{
    info_msg("Setting data file to %s", fnam.c_str());
    the_daq_manager->set_data_file(fnam);
}

int start_run(int run_type, int num_events, int sample_size, DataReceiver *D)
{
    ConfigFactory &cfg = ConfigFactory::instance();
    DAQConfig *daq = dynamic_cast<DAQConfig *>(cfg.get("daq"));
    extern AlibavaMonitor __monitor__;
    daq->nevts(num_events);
    daq->sample_size(sample_size);
    RunManager::RunType the_type;
    int rc;

    switch (run_type)
    {
        case 0:
            the_type = RunManager::Pedestal;
            break;
        case 1:
            the_type = RunManager::RadSource;
            break;
        case 2:
            the_type = RunManager::Laser;
            break;
        case 3:
            the_type = RunManager::LaserSync;
            break;
        case 4:
            the_type = RunManager::Calibration;
            break;
        default:
            std::cout << "Unknown Run type" << std::endl;
            return -1;
    }
    __monitor__.setRunTypeVal(the_type);
    the_daq_manager->set_run_type(the_type);
    rc = the_daq_manager->prepare_for_run();
    if (rc)
        return rc;

    if (D)
        the_daq_manager->get_board()->set_data_receiver(D);

    the_daq_manager->acquire_thrd(the_daq_manager, false);

    if (D)
        the_daq_manager->get_board()->remove_data_receiver(D);

    the_daq_manager->stop_run();



    return 0;
}

